@@ -211,6 +211,8 @@ class PROJ_GCC_DLL CoordinateOperation : public common::ObjectUsage,
 
     PROJ_DLL bool hasBallparkTransformation() const;
 
+    PROJ_DLL bool requiresPerCoordinateInputTime() const;
+
     PROJ_DLL static const std::string OPERATION_VERSION_KEY;
 
     PROJ_DLL CoordinateOperationNNPtr normalizeForVisualization() const;
@@ -246,6 +248,8 @@ class PROJ_GCC_DLL CoordinateOperation : public common::ObjectUsage,
         const std::vector<metadata::PositionalAccuracyNNPtr> &accuracies);
     PROJ_INTERNAL void setHasBallparkTransformation(bool b);
 
+    PROJ_INTERNAL void setRequiresPerCoordinateInputTime(bool b);
+
     PROJ_INTERNAL void
     setSourceCoordinateEpoch(const util::optional<common::DataEpoch> &epoch);
     PROJ_INTERNAL void
@@ -389,6 +393,7 @@ class PROJ_GCC_DLL GeneralParameterValue : public util::BaseObject,
 
     friend class Conversion;
     friend class SingleOperation;
+    friend class PointMotionOperation;
     PROJ_INTERNAL virtual void _exportToWKT(io::WKTFormatter *formatter,
                                             const MethodMapping *mapping)
         const = 0; // throw(io::FormattingException)
@@ -726,6 +731,10 @@ class PROJ_GCC_DLL SingleOperation : virtual public CoordinateOperation {
     createOperationParameterValueFromInterpolationCRS(int methodEPSGCode,
                                                       int crsEPSGCode);
 
+    PROJ_INTERNAL static void
+    exportToPROJStringChangeVerticalUnit(io::PROJStringFormatter *formatter,
+                                         double convFactor);
+
   private:
     PROJ_OPAQUE_PRIVATE_DATA
     SingleOperation &operator=(const SingleOperation &other) = delete;
@@ -1319,6 +1328,13 @@ class PROJ_GCC_DLL Conversion : public SingleOperation {
         const common::Angle &centerLong, const common::Length &falseEasting,
         const common::Length &falseNorthing);
 
+    PROJ_DLL static ConversionNNPtr createLocalOrthographic(
+        const util::PropertyMap &properties, const common::Angle &centerLat,
+        const common::Angle &centerLong,
+        const common::Angle &azimuthInitialLine, const common::Scale &scale,
+        const common::Length &falseEasting,
+        const common::Length &falseNorthing);
+
     PROJ_DLL static ConversionNNPtr createAmericanPolyconic(
         const util::PropertyMap &properties, const common::Angle &centerLat,
         const common::Angle &centerLong, const common::Length &falseEasting,
@@ -1678,6 +1694,12 @@ class PROJ_GCC_DLL Transformation : public SingleOperation {
         const common::Angle &offsetLong, const common::Length &offsetHeight,
         const std::vector<metadata::PositionalAccuracyNNPtr> &accuracies);
 
+    PROJ_DLL static TransformationNNPtr createCartesianGridOffsets(
+        const util::PropertyMap &properties, const crs::CRSNNPtr &sourceCRSIn,
+        const crs::CRSNNPtr &targetCRSIn, const common::Length &eastingOffset,
+        const common::Length &northingOffset,
+        const std::vector<metadata::PositionalAccuracyNNPtr> &accuracies);
+
     PROJ_DLL static TransformationNNPtr createVerticalOffset(
         const util::PropertyMap &properties, const crs::CRSNNPtr &sourceCRSIn,
         const crs::CRSNNPtr &targetCRSIn, const common::Length &offsetHeight,
@@ -1691,10 +1713,10 @@ class PROJ_GCC_DLL Transformation : public SingleOperation {
     PROJ_PRIVATE :
         //! @cond Doxygen_Suppress
         PROJ_INTERNAL const std::string &
-        getNTv2Filename() const;
+        getPROJ4NadgridsCompatibleFilename() const;
 
-    PROJ_FOR_TEST std::vector<double>
-    getTOWGS84Parameters() const; // throw(io::FormattingException)
+    PROJ_FOR_TEST std::vector<double> getTOWGS84Parameters(
+        bool canThrowException) const; // throw(io::FormattingException)
 
     PROJ_INTERNAL const std::string &getHeightToGeographic3DFilename() const;
 
@@ -1765,8 +1787,61 @@ class PROJ_GCC_DLL PointMotionOperation : public SingleOperation {
     PROJ_DLL ~PointMotionOperation() override;
     //! @endcond
 
+    PROJ_DLL const crs::CRSNNPtr &sourceCRS() PROJ_PURE_DECL;
+
+    PROJ_DLL CoordinateOperationNNPtr inverse() const override;
+
+    PROJ_DLL static PointMotionOperationNNPtr
+    create(const util::PropertyMap &properties, const crs::CRSNNPtr &crsIn,
+           const OperationMethodNNPtr &methodIn,
+           const std::vector<GeneralParameterValueNNPtr> &values,
+           const std::vector<metadata::PositionalAccuracyNNPtr>
+               &accuracies); // throw InvalidOperation
+
+    PROJ_DLL static PointMotionOperationNNPtr
+    create(const util::PropertyMap &propertiesOperation,
+           const crs::CRSNNPtr &crsIn,
+           const util::PropertyMap &propertiesOperationMethod,
+           const std::vector<OperationParameterNNPtr> &parameters,
+           const std::vector<ParameterValueNNPtr> &values,
+           const std::vector<metadata::PositionalAccuracyNNPtr>
+               &accuracies); // throw InvalidOperation
+
+    PROJ_DLL PointMotionOperationNNPtr substitutePROJAlternativeGridNames(
+        io::DatabaseContextNNPtr databaseContext) const;
+
+    PROJ_PRIVATE :
+        //! @cond Doxygen_Suppress
+        PROJ_INTERNAL PointMotionOperationNNPtr
+        shallowClone() const;
+
+    PROJ_INTERNAL PointMotionOperationNNPtr
+    cloneWithEpochs(const common::DataEpoch &sourceEpoch,
+                    const common::DataEpoch &targetEpoch) const;
+
+    PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
+        const override; // throw(io::FormattingException)
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    //! @endcond
+
+  protected:
+    PROJ_INTERNAL PointMotionOperation(
+        const crs::CRSNNPtr &crsIn, const OperationMethodNNPtr &methodIn,
+        const std::vector<GeneralParameterValueNNPtr> &values,
+        const std::vector<metadata::PositionalAccuracyNNPtr> &accuracies);
+    PROJ_INTERNAL PointMotionOperation(const PointMotionOperation &other);
+    INLINED_MAKE_SHARED
+
+    PROJ_INTERNAL CoordinateOperationNNPtr _shallowClone() const override;
+
   private:
-    PointMotionOperation(const PointMotionOperation &) = delete;
+    PointMotionOperation &operator=(const PointMotionOperation &) = delete;
 };
 
 // ---------------------------------------------------------------------------
@@ -2049,6 +2124,11 @@ class PROJ_GCC_DLL CoordinateOperationFactory {
         const coordinates::CoordinateMetadataNNPtr &targetCoordinateMetadata,
         const CoordinateOperationContextNNPtr &context) const;
 
+    PROJ_DLL std::vector<CoordinateOperationNNPtr> createOperations(
+        const coordinates::CoordinateMetadataNNPtr &sourceCoordinateMetadata,
+        const coordinates::CoordinateMetadataNNPtr &targetCoordinateMetadata,
+        const CoordinateOperationContextNNPtr &context) const;
+
     PROJ_DLL static CoordinateOperationFactoryNNPtr create();
 
   protected:
